SystemOrganization addCategory: #'System-Settings-Browser'!
SystemOrganization addCategory: #'System-Settings-Core'!
SystemOrganization addCategory: #'System-Settings-Filter'!

!Color class methodsFor: '*System-Settings-Browser' stamp: 'AlainPlantec 3/5/2010 09:33'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForColor! !

!AbstractFont class methodsFor: '*system-settings-browser' stamp: 'alain.plantec 3/18/2009 14:49'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForFont! !

!FilePath class methodsFor: '*system-settings' stamp: 'AlainPlantec 9/3/2010 17:28'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode  inputWidgetForFileOrDirectoryWithAction: #chooseFilePath.! !

!Association methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 3/24/2009 23:20'!
settingFixedDomainValueNodeFrom: aSettingNode 
	^ aSettingNode fixedDomainValueNodeForAssociation: self! !

!StrikeFont methodsFor: '*system-settings-browser' stamp: 'AlainPlantec 1/30/2010 14:48'!
settingStoreOn: aStream 
	aStream nextPut: $(.
	aStream nextPutAll: self class name.
	aStream nextPut: Character space.
	aStream nextPutAll: 'familyName: ''', self familyName, ''' pointSize: ', self pointSize asString, ' emphasized: ', self emphasis asString.
	aStream nextPut: $)
! !

!Number class methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 3/18/2009 15:09'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForNumber! !

!FileDirectory class methodsFor: '*system-settings' stamp: 'AlainPlantec 9/3/2010 17:32'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForFileOrDirectoryWithAction: #chooseFileDirectory.! !

!Point class methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 4/21/2009 09:57'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForPoint! !

!LogicalFont methodsFor: '*system-settings-browser' stamp: 'alain.plantec 3/19/2009 10:02'!
settingStoreOn: aStream 
	aStream nextPut: $(.
	aStream nextPutAll: self class name.
	aStream nextPut: Character space.
	aStream nextPutAll: 'familyName: '.
	self familyName settingStoreOn: aStream.
	aStream nextPut: Character space.
	aStream nextPutAll: 'fallbackFamilyNames: '.
	self fallbackFamilyNames settingStoreOn: aStream.
	aStream nextPut: Character space.
	aStream nextPutAll: 'pointSize: '.
	self pointSize settingStoreOn: aStream.
	aStream nextPut: Character space.
	aStream nextPutAll: 'stretchValue: '.
	self stretchValue settingStoreOn: aStream.
	aStream nextPut: Character space.
	aStream nextPutAll: 'weightValue: '.
	self weightValue settingStoreOn: aStream.
	aStream nextPut: Character space.
	aStream nextPutAll: 'slantValue: '.
	self slantValue settingStoreOn: aStream.
	aStream nextPut: $)! !

!LogicalFont methodsFor: '*system-settings-browser' stamp: 'AlainPlantec 12/18/2009 08:55'!
withSizeIncrementedBy: anInteger
	^ LogicalFont 
		familyName: self familyName 
		fallbackFamilyNames: self fallbackFamilyNames
		pointSize: self pointSize + anInteger
		stretchValue: self stretchValue
		weightValue: self weightValue
		slantValue: self slantValue ! !

MorphTreeModel subclass: #SettingBrowser
	instanceVariableNames: 'viewedPackages roots status searchedText treeHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Browser'!

SettingBrowser class
	instanceVariableNames: 'searchedTextList regexpSearch'!

!SettingBrowser commentStamp: 'AlainPlantec 2/25/2011 16:00' prior: 0!
<help>

!! The User interface
A SettingBrowser allows the browsing as well as the editing of setting values.
For each setting, a label and an input widget allowing the change of the setting value are shown.

!!!! Browsing standards settings
In order to open a setting browser one can also use the expression below:
--------------------
SettingBrowser open
--------------------
It opens a SettingBrowser for all settings declared with the standard 'systemsettings' pragma keyword. 
To see how these settings are declared in the image, you can browse all senders of #systemsettings:
--------------
SystemNavigation new  browseAllCallsOn: #systemsettings
--------------

!!!! Browsing application specific settings
For specific applications, other pragma keywords can be used. These keywords can be passed as follow:
--------------------
(SettingBrowser forKeywords: #('blob')) open.
--------------------
Then, only settings which are declared with the keyword 'blob' are viewed. 
Here is an example of such a setting declared in the class side of a class BlobSettings 
--------------------
BlobSettings class>>blobSettingOn: aBuilder
	<blob>
	(aBuilder group: #blobEditing) 
		label: 'Editing' translated;
		parent: #blobBrowsing; 
		description: 'All settings concerned with blob editing' translated;
		with: [	
			(aBuilder setting: #color)	...
--------------------
The expression 'SettingBrowser open' is then equivalent to the '(SettingBrowser forKeywords: #('systemsettings')) open' expression.

!!!! Filtering
The SettingBrowser has a filtering functionality to limitate the number of settings. You can enter a token in the input field of the toolbar in order to show only settings that are matching the token.
Two filtering modes are allowed:  with a substring or with a regular expression.
- with a substring: only settings which name, description, pragma class or method selector includes this substring are viewed; 
- with a regular expression: the same as with a substring except that the input is used as a regular expression. This filtering is applied if the Regexp checkbox is checked.

!! How to declare a setting
A setting is declared with a method class side. This kind of method takes a builder as argument and a standard setting is tagged with the <systemsettings> pragma. The builder argument serves as a facade for the declaration.

!!!!A simple boolean setting example

Let's start with a setting declaration example:
CodeHolderSystemSettings class>>caseSensitiveFindsSettingsOn: aBuilder
	<systemsettings>
	(aBuilder setting: #caseSensitiveFinds) 
		label: 'Case sensitive search' translated;
		description: 'If true, then the "find" command in text will always make its searches in a case-sensitive fashion' translated;
		setSelector: #caseSensitiveFinds:;
		getSelector: #caseSensitiveFinds;
		target: TextEditor;
		parent: #codeEditing.

For this setting to be declared, we make the asumption that we have TextEditor class>>caseSensitiveFinds and TextEditor class>>caseSensitiveFinds: methods in order to change the preference value. 
To declare a setting, just send #setting: to the builder with its identifier, a Symbol passed as argument. It creates a setting node. Then you can set the label, the description with #label: and #description sent to the newly created setting node. You also have to set the selectors for setting and getting the preference value as well as the target to which these accessors are sent  (often a class). This is done by sending respectively, #setSelector:, #getSelector: and #target: to the setting node.
Because all settings are organized in trees we need a way to indicate what is the position of the setting node in the overall setting trees list. In fact it can be done two ways. The first way is to use the #parent: message (A second possibility is to declare a subtree in one method, it is explained later in this documentation).The #parent: message is send for non root settings. #parent takes the identifier of the parent setting as argument.

You may notice that in this example,  if we don't take into account the $: at the end of the setting accessor, the getting and the setting accessors are the same. This is often the case. You can simply set the setter and the getter by sending the #selector: message to the setting node. Thus the declaration is simplified as follow:
CodeHolderSystemSettings class>>caseSensitiveFindsSettingsOn: aBuilder
	<systemsettings>
	(aBuilder setting: #caseSensitiveFinds) 
		label: 'Case sensitive search' translated;
		description: 'If true, then the "find" command in text will always make its searches in a case-sensitive fashion' translated;
		selector: #caseSensitiveFinds;
		target: TextEditor;
		parent: #codeEditing.

You may also notice that the identifier of the setting is then the same as the selector. In this case, you can omit to set the selector because by default, the identifier is used as the selector for getting the preference value and the identifier concatenated with a $: at the end is used as the setting selector. Thus the declaration is simplified again as follow:
CodeHolderSystemSettings class>>caseSensitiveFindsSettingsOn: aBuilder
	<systemsettings>
	(aBuilder setting: #caseSensitiveFinds) 
		label: 'Case sensitive search' translated;
		description: 'If true, then the "find" command in text will always make its searches in a case-sensitive fashion' translated;
		target: TextEditor;
		parent: #codeEditing.

Finally, if you decide to declare a setting directly in the class which implements the selector, the target information can be also omitted because the class in which the setting is declared is used by default. Thus the simplest declaration can be:
TextEditor class>>caseSensitiveFindsSettingsOn: aBuilder
	<systemsettings>
	(aBuilder setting: #caseSensitiveFinds) 
		label: 'Case sensitive search' translated;
		description: 'If true, then the "find" command in text will always make its searches in a case-sensitive fashion' translated;
		parent: #codeEditing.

!!!!Declaring a subtree in one single method
Directly declaring a sub-tree of settings in one method is also possible. Then, typically, a root group is declared for the application settings and the children settings themselves are also declared within the same method as in the example below in which #formatCommentWithStatements and #indentString are directly declared as children of the #configurableFormatter setting:

RBConfigurableFormatter class>>settingsOn: aBuilder
	<systemsettings>	
	(aBuilder group: #configurableFormatter)
		target: self;
		parent: #refactoring;
		label: 'Configurable Formatter' translated;
		description: 'Settings related to the formatter' translated;
		with: [
			(aBuilder setting: #formatCommentWithStatements)
				label: 'Format comment with statements' translated.
			(aBuilder setting: #indentString)
				label: 'Indent string' translated]

!!!!Optional setting
Any setting can have children. In the case where a boolean setting is used as a parent, then, its children are chown only if the parent preference value is true.

!!!!Range setting
You send the #range: message to the builder instead of the #setting: message. In addition, you send the #range: message to the setting with an interval as argument in order to indicate the valid range.
screenMarginSettingOn: aBuilder
	<systemsettings>
	(aBuilder range: #fullScreenMargin)
		target: SystemWindow;
		parent: #windows;
		label: 'Full screen margin' translated;
		description: 'Specify the amount of space that is let around a windows when it''s opened fullscreen' translated;
		range: (-5 to: 100).

!!!!List setting
For this kind of setting, the SettingBrowser will show a drop list. Here is an example for the window position strategy. Notice that the setting is declared by sending the #pickOne message to the builder. Notice also that the list of valid values are given by sending the #domainValues: message to the setting. The argument is an array of association. Each association key is the label that is shown and the corresponding association value gives the value that will be assigned to the preference.
windowPositionStrategySettingsOn: aBuilder
	<systemsettings>
	(aBuilder pickOne: #usedStrategy) 
		label: 'Window position strategy' translated;
		target: RealEstateAgent;
		domainValues: {
			'Reverse Stagger' translated -> #staggerFor:initialExtent:world:. 
			'Cascade' translated -> #cascadeFor:initialExtent:world:. 
			'Standard' translated -> #standardFor:initialExtent:world:};

!!!!Launcher
A launcher is a particular setting. It allows to launch a script directly from the setting browser. Imagine that you have changed some settings and that you need to evaluate a script in order to update some other objets. It can be used also to configurate globally a package of the entire image.
As an example, in order to use True Type Fonts, the system must be updated by collecting all the available TT fonts. This can be done by evaluating the following expression:
-------------
FreeTypeFontProvider current updateFromSystem
-------------
In order to be able to launch this script from the setting browser, you have to declare a launcher. For example, look-at how the script for the TT fonts is declared in GraphicFontSettings class >> #standardFontsSettingsOn:.

GraphicFontSettings class >> #standardFontsSettingsOn:
	<systemsettings>
	(aBuilder group: #standardFonts)
		...
		(aBuilder launcher: #updateFromSystem)
				order: 1; 
				target: FreeTypeFontProvider;
				targetSelector: #current;
				description: 'Update available fonts by scanning the current system';
				script: #updateFromSystem;
				label: 'Update fonts from system' translated.

Notice that you send #launcher: to the builder in order to create the setting node, then you send #script: to the created node with the selector of the script passed as argument.
</help>

Implementation details
See also SettingNode, SettingTree, SettingNodeBuilder and SettingTreeBuilder classes.

Instance Variables
	roots:		<Collection of SettingTreeNode>
	searchedText:		<String>
	status:		<WriteStream>
	collector:		<SettingCollector>

roots
	- the roots of the viewed setting trees

searchedText
	- the string used as a filter token or regexp in order to select viewed settings

status
	- a WriteStream used in order to store useful information which are shown when no current item is selected

collector
	- The SettingCollector which has the responsibility to collect settings and to build viewed trees
!

SettingBrowser class
	instanceVariableNames: 'searchedTextList regexpSearch'!

!SettingBrowser class methodsFor: 'private helpers' stamp: 'AlainPlantec 9/17/2011 21:29'!
allSettingsState
	"
	Workspace openContents: self allSettingsState label: 'Current Setting assignments' 
	"
	| declList |
	declList := (self currentNodeList select: [ :node | node item hasValue])
		collect: [ :node | node item ].
	^ String streamContents: [:s | declList do: [:d | d settingStoreOn: s] ]! !

!SettingBrowser class methodsFor: 'opening' stamp: 'AlainPantec 1/7/2011 08:19'!
browse: aCollectionOfKeywords
	"open a SettingBrowser view which is able to browse all settings 
	that are declared with a pragma keyword which is at least
	one of the string contained in aCollectionOfKeywords
	example:
	(SettingBrowser browse: #('systemsettings'))
	browse all settings which make use of the pragma keyword 'systemsettings'
	
	Note that the default setting pragma keyword is 'systemsettings'
	"
	(self forKeywords: aCollectionOfKeywords) open! !

!SettingBrowser class methodsFor: 'private helpers' stamp: 'AlainPlantec 9/14/2011 16:10'!
currentNodeList
	^ (SettingTree new  acceptableKeywords: self settingsKeywords) nodeList! !

!SettingBrowser class methodsFor: 'opening' stamp: 'AlainPantec 1/7/2011 08:19'!
forKeywords: aCollectionOfKeywords
	"create a new SettingBrowser initialized with a set of pragma keywords"
	^ self new acceptableKeywords: aCollectionOfKeywords! !

!SettingBrowser class methodsFor: 'accessing' stamp: 'AlainPlantec 9/17/2011 18:32'!
initialExtent
	"Initial extent of the SettingBrowser window"
	^ 900@600
		! !

!SettingBrowser class methodsFor: 'world menu' stamp: 'FernandoOlivero 4/12/2011 10:15'!
menuCommandOn: aBuilder
	<worldMenu>
	(aBuilder item: #'Settings')
		parent: #System;
		order: 1;
		icon:  self theme  smallConfigurationIcon;
		action: [SettingBrowser open]; 
		help: 'Opens a SystemSettingBrowser which allows you to alter many system settings'! !

!SettingBrowser class methodsFor: 'opening' stamp: 'alain.plantec 4/13/2009 14:33'!
open
	"Open a SettingBrowser view"
	^ self new open! !

!SettingBrowser class methodsFor: 'updating' stamp: 'AlainPlantec 11/30/2009 10:22'!
refreshAllSettingBrowsers
	self allSubInstances do: [:sb | sb updateList]! !

!SettingBrowser class methodsFor: 'accessing' stamp: 'AlainPantec 1/7/2011 08:15'!
regexpSearch
	"Is the regexp filtering used"
	^ regexpSearch ifNil: [regexpSearch := false]! !

!SettingBrowser class methodsFor: 'accessing' stamp: 'AlainPantec 1/7/2011 08:16'!
regexpSearch: aBoolean
	"Set to true if the regexp filtering is to be used"

	regexpSearch := aBoolean! !

!SettingBrowser class methodsFor: 'accessing' stamp: 'AlainPantec 1/7/2011 08:16'!
searchedTextList
	"Keep a list of previously entered filtering tokens"
	^ searchedTextList ifNil: [searchedTextList := OrderedCollection new]! !

!SettingBrowser class methodsFor: 'accessing' stamp: 'AlainPlantec 9/14/2011 16:05'!
settingsKeywords 
	^ ((PragmaCollector new filter: [:prg | prg keyword = #settingPragmaProcessor]) reset) collect: [:p | p method selector].
! !

!SettingBrowser class methodsFor: 'private helpers' stamp: 'AlainPlantec 9/17/2011 21:29'!
viewAllSettings
	"self viewAllSettings"
	Workspace openContents: self allSettingsState label: 'Current Setting assignments' 
! !

!SettingBrowser class methodsFor: 'help system' stamp: 'AlainPlantec 1/5/2011 13:03'!
wikiStyleHelp
	<wikiStyleHelp: #'The setting browser'>
	^ self comment! !

!SettingBrowser methodsFor: 'menu' stamp: 'AlainPlantec 1/29/2010 21:22'!
about
	self inform: 'Here you can browse, query and change all system settings' translated! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 11:44'!
acceptableKeywords: aCollectionOfRegExps 
	self treeHolder acceptableKeywords: aCollectionOfRegExps ! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/23/2009 15:03'!
addToSearchedTextList: aString
	(self searchedTextList includes: aString) 
		ifFalse: [self searchedTextList size > 5 ifTrue: [self searchedTextList removeFirst].
		self searchedTextList add: aString]! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/2/2009 14:12'!
addToStatus: aText 
	self status nextPutAll: aText.
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 11:05'!
allPackagesWithSettings
	^( self treeHolder nodeList collect: [:n | n receiverPackage]) asSet! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 1/30/2010 10:18'!
applyFilters
	roots := nil.
	self searchedText
		ifNotEmpty: [self updateAccordingTo: self textFilter].
	self viewedPackages
		ifNotEmpty: [ | filter |
			filter := SettingPackageFilter new
			packages: self viewedPackages.
			self updateAccordingTo: filter].
	self updateList
! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 10/7/2011 10:08'!
buildWindowIn: aWindow 
	| statusView toolBar treeMorph toolBarY gap menuBar packageListView statusViewHeight |
	treeMorph := self treeMorphIn: aWindow.
	statusView := self statusViewIn: aWindow.
	statusViewHeight := 100.
	toolBar := self toolBarIn: aWindow.
	gap := 3.
	packageListView := self packageListViewIn: aWindow.
	toolBarY := toolBar minExtent y + gap.
	aWindow 
		addMorph: toolBar
		fullFrame: (LayoutFrame 
				fractions: (0 @ 0 corner: 1 @ 0)
				offsets: (0 @ (gap ) corner: 0 @ (toolBarY + gap))).
	aWindow 
		addMorph: treeMorph
		fullFrame: (LayoutFrame 
				fractions: (0 @ 0 corner: 1 @ 0.99)
				offsets: (0 @ (toolBarY + gap + gap) corner: 0 @ statusViewHeight negated)).
	aWindow 
		addMorph: statusView
		fullFrame: (LayoutFrame 
				fractions: (0 @ 0.99 corner: 0.75 @ 1)
				offsets: (0 @ statusViewHeight negated corner: 0 @ 0)).
	aWindow 
		addMorph: packageListView
		fullFrame: (LayoutFrame 
				fractions: (0.75 @ 0.99 corner: 1 @ 1)
				offsets: (0 @ statusViewHeight negated corner: 0 @ 0)).
	^ aWindow! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 1/29/2010 21:08'!
changePackageSet: aSet 
	aSet ~= self viewedPackages
		ifTrue: [self viewedPackages: aSet.
			self applyFilters].
	^ true! !

!SettingBrowser methodsFor: 'updating' stamp: 'alain.plantec 4/23/2009 14:59'!
changeSearchedText: aString 
	self searchedText ~= aString
		ifTrue: [self searchedText: aString.
			self applyFilters].
	^ true ! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 10/11/2011 14:59'!
choosePackagesIn: aWindow 
	| result list |
	list := MorphTreeModel new
				beCheckList;
				autoMultiSelection: true;
				rootItems: (self allPackagesWithSettings asArray sort: [:a :b | a packageName < b packageName]);
				 wrapBlockOrSelector: #packageName.
	result := list openDialogWindowIn: aWindow title: 'Choose packages' selectedtems: self viewedPackages.
	result ifNotNil: [self changePackageSet: result]! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 9/17/2011 21:33'!
closePathFrom: aSetting 
	(self settingNodeOfDeclaration: aSetting)
		ifNotNil: [:node | self changed: {#rootNodes. #closeItemPath}, (node path collect: [:n | n withoutListWrapper])]! !

!SettingBrowser methodsFor: 'menu' stamp: 'AlainPlantec 12/1/2009 15:51'!
doubleClick
	self selectedNode ifNotNil: [:node | node browseDeclaration]! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 9/17/2011 21:33'!
expandAll
	self changed: #(#rootNodes #expandAll).
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 13:51'!
getDescription
	^ (self selectedNode)
		ifNil: [self status contents ifEmpty: ['Hit return in text fields to accept the input' translated]]
		ifNotNil: [:sel | (Text string: sel item label attribute: TextEmphasis bold), String cr, sel item description]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 1/30/2010 10:10'!
getViewedPackageNames
	^ Text
		streamContents: [:s | self viewedPackages
				ifEmpty: [s nextPutAll: 'All packages' translated]
				ifNotEmpty: [:selections | 
					(selections asSet = self allPackagesWithSettings asSet)
						ifTrue: [s nextPutAll: 'All packages']
						ifFalse: [selections
								do: [:package | 
									s nextPutAll: package packageName.
									s nextPut: Character space]]]]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'IgorStasenko 4/26/2011 15:57'!
helpDeclaringAndManagingSettings
	"Open up a workspace with explanatory info in it about the Setting Browser"
	Smalltalk tools workspace new
		contents: "SettingManager comment;" 'To be documented here later';
		openLabel: 'Declaring and managing settings' translated! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 21:23'!
helpMenuIn: aWindow
	| helpMenu |
	helpMenu := aWindow newMenu.
	helpMenu
		addToggle: 'About' translated
		target: self
		selector: #about.
	helpMenu lastItem font: aWindow theme menuFont.
	helpMenu
		addToggle: 'Declaring and managing settings' translated
		target: self
		selector: #helpDeclaringAndManagingSettings.
	helpMenu lastItem font: aWindow theme menuFont.
	^ helpMenu
! !

!SettingBrowser methodsFor: 'user interface' stamp: 'alain.plantec 9/21/2009 09:40'!
initialExtent
	^ self class initialExtent! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 12/1/2009 09:10'!
isRendered
	| renderingMorphs |
	renderingMorphs := SystemWindow allSubInstances select: [:sw | sw model = self].
	^ renderingMorphs anySatisfy: [:m | m owner notNil]! !

!SettingBrowser methodsFor: 'menu' stamp: 'AlainPlantec 2/25/2011 17:07'!
keyStroke: anEvent from: aTreeMorph
	| c |
	c := anEvent keyCharacter.
	self selectedNode
		ifNotNil: [:current | current keyStroke: anEvent from: aTreeMorph.
			current settingDeclaration hasDefault
				ifTrue: [c = $d
						ifTrue: [^ self menuSetToDefault: current]].
			current settingDeclaration hasEditableList
				ifTrue: [c = $e
						ifTrue: [^ self menuEmptyList: current]]]! !

!SettingBrowser methodsFor: 'menu' stamp: 'AlainPlantec 3/10/2010 07:51'!
menu: menu shifted: b 
	"Set up the menu to apply to the receiver's, 
	honoring the #shifted boolean"
	self selectedNode ifNotNil: [:current | current menu: menu shifted: b].
	menu addLine.
	(self selectedNode)
		ifNotNil: [:current | 
			current settingDeclaration hasDefault
				ifTrue: [menu
						add: 'Set to default (d)' translated
						target: self
						selector: #menuSetToDefault:
						argument: current].
			current settingDeclaration hasEditableList
				ifTrue: [menu
						add: 'Empty list (e)' translated
						target: self
						selector: #menuEmptyList:
						argument: current]].
	^ menu! !

!SettingBrowser methodsFor: 'menu' stamp: 'alain.plantec 10/19/2009 14:49'!
menuEmptyList: aTreeNode
	| v |
	(v := aTreeNode settingDeclaration) hasEditableList 
		ifTrue: [v emptyList. self updateList]! !

!SettingBrowser methodsFor: 'menu' stamp: 'alain.plantec 10/19/2009 14:17'!
menuSetToDefault: aTreeNode
	| v |
	(v := aTreeNode settingDeclaration) hasDefault
		ifTrue: [(self confirm: 'Set to default value ?' translated)
				ifTrue: [v setToDefault. self updateList]]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 4/29/2010 17:51'!
newTreeHolder
	^ SettingTree new  acceptableKeywords: self settingsKeywords! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 10/10/2011 11:31'!
open
	
	Cursor wait
		showWhile: [ | window |window := StandardWindow new model: self.
			window title: self windowTitle.
			self buildWindowIn: window.
			window themeChanged.
			window openInWorld.
			(window findDeeplyA: self treeMorphClass)  takeKeyboardFocus. 
			^ window]! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 9/17/2011 21:33'!
openPathFrom: aSetting
	(self settingNodeOfDeclaration: aSetting) ifNotNil: [:node |
		self changed: {#rootNodes. #openItemPath}, (node path collect: [:n | n withoutListWrapper])]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 21:10'!
packageListViewIn: aWindow
	^ aWindow
				newTextEditorFor: self
				getText: #getViewedPackageNames
				setText: #setViewedPackageNames:.
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 3/27/2009 23:04'!
pragmaErrorOccured: aPragmaError
	self setDescription: aPragmaError message, String cr! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/23/2009 15:01'!
regexpSearch
	^ self class regexpSearch! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/23/2009 15:02'!
regexpSearch: aBoolean
	self class regexpSearch: aBoolean.
	self applyFilters 
! !

!SettingBrowser methodsFor: 'initialize-release' stamp: 'AlainPlantec 1/29/2010 21:10'!
release
	treeHolder release.
	treeHolder := nil.
	roots := nil.
	status := nil.
	searchedText := nil.
	viewedPackages := nil.
	super release.! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 2/4/2010 17:15'!
rootNodes
	^ roots ifNil: [roots := self treeHolder settingTreeRoots]
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/8/2009 08:58'!
searchedText
	^ searchedText ifNil: [searchedText := '']! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 3/23/2009 18:38'!
searchedText: aString
	searchedText := aString.
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/20/2009 09:19'!
searchedTextList
	^ self class searchedTextList! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 11/18/2009 08:28'!
selection: aSelectionHolder
	super selection: aSelectionHolder.
	self setDescription: ''.
	self changed: #getDescription
! !

!SettingBrowser methodsFor: 'menu' stamp: 'AlainPlantec 11/24/2009 11:44'!
setAllToDefault
	(self confirm: 'Set all to default value ?' translated) ifTrue: [
		self treeHolder nodeList do: [:node | node item hasDefault ifTrue: [node item setToDefault]].
		self updateList]! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/2/2009 14:12'!
setDescription: aText 
	self status reset.
	self addToStatus: aText.
	self changed: #getDescription! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 21:10'!
setViewedPackageNames: aText 
	| allViewed |
	allViewed := Set new.
	aText asString subStrings
		do: [:sub | (PackageOrganizer default
						packageNamed: sub
						ifAbsent: [])
				ifNotNil: [:pkg | allViewed add: pkg]].
	self changePackageSet: allViewed.
	self changed: #getViewedPackageNames
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 11:44'!
settingNodeOfDeclaration: aSettingDeclaration
	^ self treeHolder deeplyDetect: [:n | n settingDeclaration = aSettingDeclaration]! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 11/27/2009 22:08'!
settingValueChanged: aSetting 
	aSetting realValue = true
		ifTrue: [self openPathFrom: aSetting].
	aSetting realValue = false
		ifTrue: [self closePathFrom: aSetting].
	aSetting changed: #realValue.
	aSetting changed: #fontButtonLabel.
	aSetting changed: #index.
	aSetting changed: #content ! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 9/14/2011 16:06'!
settingsKeywords 
	^ self class settingsKeywords! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/2/2009 14:11'!
status
	^ status ifNil: [status := String new writeStream]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'alain.plantec 3/23/2009 18:38'!
statusViewIn: aMorph
	| statusView |
	statusView := aMorph
				newTextEditorFor: self
				getText: #getDescription
				setText: #setDescription:.
	^ statusView
 ! !

!SettingBrowser methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:15'!
taskbarIcon
	^ self theme smallConfigurationIcon! !

!SettingBrowser methodsFor: 'updating' stamp: 'alain.plantec 4/23/2009 15:01'!
textFilter
	^ self regexpSearch
		ifFalse: [SettingTokenFilter new token: self searchedText]
		ifTrue: [SettingRegexpFilter new regexp: self searchedText] ! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 2/13/2010 16:55'!
toolBarIn: aMorph 
	| toolBar |
	toolBar := aMorph newToolbar: {
	aMorph
		newButtonFor: self
		getState: nil
		action: #expandAll
		arguments: nil
		getEnabled: nil
		label: 'Expand all' translated
		help: 'Expand all tree nodes' translated.
	aMorph newToolSpacer.
	aMorph newToolSpacer.
	aMorph newLabel: 'Search for: ' translated.
	(EditableDropListMorph
				on: self
				list: #searchedTextList
				selected: #searchedText
				changeSelected: #changeSearchedText:
				useIndex: false 
				addToList: #addToSearchedTextList:
				class: String
				getEnabled: nil) hResizing: #spaceFill; 
				ghostText: 'Hit return to accept' translated.
	aMorph
			newCheckboxFor: self
			getSelected: #regexpSearch
			setSelected: #regexpSearch:
			getEnabled: nil
			label: 'Regexp' 
			help: 'Filter according to a regular expression' translated.
	aMorph newToolSpacer.	
	aMorph newToolSpacer.
	(aMorph
				newButtonFor: self
				getState: nil
				action: #choosePackagesIn:
				arguments: {aMorph}
				getEnabled: nil
				label: 'Choose packages' translated 
				help: 'Select packages' translated).
	aMorph newToolSpacer}.
	^ toolBar! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 11/18/2009 08:31'!
treeChanged: anAnnounce
	roots := nil.
	self selection: nil.
	self updateList! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 11:42'!
treeHolder
	^ treeHolder
		ifNil: [treeHolder := self newTreeHolder whenChangedSend: #treeChanged: to: self; yourself]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 10/10/2011 11:31'!
treeMorphIn: aMorph 
	| treeMorph |
	treeMorph := (self treeMorphClass on: self)
				getMenuSelector: #menu:shifted:;
				keystrokeActionSelector: #keyStroke:from:;
				columns: {
					MorphTreeColumn new startWidth: 300; rowMorphGetSelector: #labelMorphFor:; yourself.
					MorphTreeColumn new rowMorphGetSelector: #inputMorphFor:;yourself
				};
				doubleClickSelector: #doubleClick;
				rowInset: 6;
				columnInset: 6;
				rowColorForEven: Color veryLightGray muchLighter;
				preferedPaneColor: Color veryLightGray lighter lighter;
				yourself.
	^ treeMorph buildContents! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 9/17/2011 21:34'!
updateAccordingTo: aFilter
	self selection: nil.
	self setDescription: ''.
	roots := nil.
	self updateList.
	[
	self rootNodes do: [:r | r updateAccordingTo: aFilter].
	self rootNodes removeAllSuchThat: [:n | (aFilter keepSettingTreeNode: n) not and: [n isEmpty]].
	]
		on: RegexSyntaxError
		do: [:ex | 
			self setDescription: 'Regular expression syntax error, see `documentation'' protocol of RxParser class for user''s guide'].
	self expandAll! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 9/17/2011 21:34'!
updateList
	self isRendered ifTrue: [
		self treeHolder updateList.
		self changed: #rootNodes.
		self changed: #getDescription.
		self changed: #getViewedPackageNames].	
! !

!SettingBrowser methodsFor: 'updating' stamp: 'alain.plantec 4/2/2009 14:12'!
updateStatusWith: aMessage
	self addToStatus: aMessage.
	self addToStatus: Character cr.
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 9/17/2011 21:25'!
viewedLabelOfSetting: aSettingDeclaration 
	| node |
	node := self settingNodeOfDeclaration: aSettingDeclaration.
	^ node 
		ifNil: [aSettingDeclaration label]
		ifNotNil: [(node allChildren notEmpty and: [aSettingDeclaration hasValue])
				ifTrue: [aSettingDeclaration label, '...'] 
				ifFalse: [aSettingDeclaration label]]! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 21:11'!
viewedPackages
	^ viewedPackages
		ifNil: [viewedPackages := Set new]! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 1/30/2010 10:18'!
viewedPackages: aCollectionOfPackagesOrNil
	viewedPackages := aCollectionOfPackagesOrNil! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 21:29'!
windowTitle
	^ 'Settings browser'! !

!Array methodsFor: '*System-Settings-Browser' stamp: 'AlainPlantec 1/31/2010 10:35'!
settingStoreOn: aStream 
	"Use the literal form if possible."

	self isLiteral
		ifTrue: 
			[aStream nextPut: $#; nextPut: $(.
			self do: 
				[:element | 
				element settingStoreOn: aStream.
				aStream space].
			aStream nextPut: $)]
		ifFalse: [super settingStoreOn: aStream]! !

MorphTreeNodeModel subclass: #SettingNode
	instanceVariableNames: 'parentName pragma contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Browser'!

!SettingNode commentStamp: 'alain.plantec 4/20/2009 21:42' prior: 0!
A SettingTreeNode is used by a SettingCollector as a node for setting trees. Built trees can be viewed by a SettingBrowser

Instance Variables
	contents:		<Collection of SettingTreeNode>
	parentNode:		<SettingTreeNode or nil>

contents
	- the children

parentNode
	- the parent or nil if the node is a root
!

!SettingNode methodsFor: 'comparing' stamp: 'alain.plantec 3/3/2009 22:46'!
<= other
	^ self asString <= other asString! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 07:33'!
allChildren
	^ self model childrenOf: self.
! !

!SettingNode methodsFor: 'menu' stamp: 'IgorStasenko 4/26/2011 15:58'!
browseDeclaration
	"open a browser on the method which defines myself"
	Smalltalk tools browse: self methodClass class selector: self selector! !

!SettingNode methodsFor: 'checking' stamp: 'AlainPlantec 12/13/2009 21:59'!
checkForUnknownTarget
	self parentNode ifNotNil: [:p | item target ifNil: [item target: p item target].
		item targetSelector ifNil: [item targetSelector: p item targetSelector]].
	self allChildren do: [:c | c checkForUnknownTarget]! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 08:52'!
contents
	(self settingDeclaration hasValue
			and: [self settingValue = false])
		ifTrue: [^ OrderedCollection new]
		ifFalse: [^ contents 
				ifNil: [contents := self allChildren.
					self settingDeclaration ordering
						ifTrue: [contents := contents asArray sort: self model sortBlock].
					contents := self model retainedNodesFromList: contents asOrderedCollection]]! !

!SettingNode methodsFor: 'menu' stamp: 'alain.plantec 4/3/2009 07:59'!
deeplyDetect: aBlock 
	(aBlock value: self) ifTrue: [^ self].
	self contents do: [:sub | (sub deeplyDetect: aBlock) ifNotNil: [:found | ^ found]].
	^ nil! !

!SettingNode methodsFor: 'menu' stamp: 'alain.plantec 4/3/2009 07:57'!
deeplySelect: aBlock in: aCollection
	(aBlock value: self) ifTrue: [aCollection add: self].
	self contents do: [:sub | sub deeplySelect: aBlock in: aCollection].
	^ aCollection! !

!SettingNode methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 16:06'!
description
	^ self settingDeclaration description! !

!SettingNode methodsFor: 'menu' stamp: 'AlainPlantec 11/24/2009 15:24'!
exploreItem
	self settingDeclaration explore! !

!SettingNode methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 16:38'!
icon
	^ self settingDeclaration ifNotNil: [:d | d icon ] ! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/11/2009 09:46'!
inputMorphFor: aContainer
	^ self settingDeclaration 
		ifNotNil: [:v | v inputMorphFor: aContainer]! !

!SettingNode methodsFor: 'menu' stamp: 'AlainPlantec 11/24/2009 15:24'!
inspectItem
	self settingDeclaration inspect! !

!SettingNode methodsFor: 'testing' stamp: 'alain.plantec 3/17/2009 09:45'!
isEmpty
	^ self contents isEmpty! !

!SettingNode methodsFor: 'menu' stamp: 'AlainPlantec 2/25/2011 17:07'!
keyStroke: anEvent from: aTreeMorph
	| c |
	c := anEvent keyCharacter.
	c = $b
		ifTrue: [self menuBrowse. ^ true].
	c = $i
		ifTrue: [self inspectItem. ^ true].
	c = $I
		ifTrue: [self exploreItem. ^ true].
	^ false
! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 14:13'!
labelMorphFor: aContainer
	^ self settingDeclaration 
		ifNotNil: [:v | v labelMorphFor: aContainer]! !

!SettingNode methodsFor: 'menu' stamp: 'AlainPlantec 3/10/2010 07:32'!
menu: menu shifted: b 
	menu
		add: 'Browse (b)' translated
		target: self
		selector: #browseDeclaration.
! !

!SettingNode methodsFor: 'menu' stamp: 'alain.plantec 3/23/2009 10:11'!
menuBrowse
	self browseDeclaration! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/3/2009 14:44'!
methodClass
	^ self pragma ifNotNil: [self pragma methodClass theNonMetaClass]! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/9/2009 07:52'!
order
	^ item order! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/9/2009 07:52'!
order: anInteger
	item order: anInteger! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/3/2009 14:46'!
package
	
	^ self methodClass ifNotNil: [:mc | PackageOrganizer default mostSpecificPackageOfClass: mc ifNone: []]! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 09:26'!
parent: aSymbol
	self parentName: aSymbol! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 09:27'!
parentName
	^ parentName
	! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 09:27'!
parentName: aSymbol
	parentName := aSymbol.
	! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 07:51'!
parentNode
	^ self model nodeNamed: self parentName! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 14:51'!
pragma
	^ pragma! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 07:38'!
pragma: aPragma
	pragma := aPragma! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:50'!
receiverPackage
	^ self settingReceiver
		ifNotNil: [PackageOrganizer default
				mostSpecificPackageOfClass: self settingReceiver class
				ifNone: []] ! !

!SettingNode methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 5/16/2011 16:14'!
selector
	^ self pragma 
		ifNil: ['']
		ifNotNil: [:p | p selector]! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 07:11'!
settingDeclaration
	^ self item! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:50'!
settingReceiver
	^ item settingReceiver! !

!SettingNode methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 23:40'!
settingValue
	^ self settingDeclaration realValue
! !

!SettingNode methodsFor: 'updating' stamp: 'AlainPlantec 1/2/2011 19:19'!
updateAccordingTo: aFilter 
	self contents do: [ :n | n updateAccordingTo: aFilter].
	self contents removeAllSuchThat: [ :n | (aFilter keepSettingTreeNode: n) not  and: [n isEmpty]]! !

!StrikeFontSet methodsFor: '*system-settings-browser' stamp: 'AlainPlantec 1/31/2010 10:45'!
settingStoreOn: aStream 
	aStream nextPut: $(.
	aStream nextPutAll: self class name.
	aStream nextPut: Character space.
	aStream nextPutAll: 'familyName: ''', self familyName, ''' size: ', self pointSize asString, ' emphasized: ', self emphasis asString.
	aStream nextPut: $)
! !

!Boolean class methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 3/18/2009 14:48'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForBoolean! !

!Object methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 3/24/2009 23:21'!
settingFixedDomainValueNodeFrom: aSettingNode 
	^ aSettingNode fixedDomainValueNodeForObject: self! !

!Object methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 3/19/2009 09:50'!
settingStoreOn: aStream
	^ self storeOn: aStream! !

Object subclass: #PragmaSetting
	instanceVariableNames: 'precondition target targetSelector name label description order icon ordering dialog allowedInStyle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!PragmaSetting commentStamp: 'AlainPlantec 1/3/2011 10:54' prior: 0!
A PragmaSetting is a holder for a setting. A setting is declared in a class method which contains a pragma (as examples, <menu> or <setting>). 
Settings can be collected on the fly by a PragmaCollector and browsed by a SettingBrowser. Settings are organized in trees.

Instance Variables
	allowedInStyle:		<Object>
	description:		<Object>
	dialog:		<Object>
	icon:		<Object>
	label:		<Object>
	name:		<Object>
	order:		<Object>
	ordering:		<Object>
	precondition:		<Object>
	target:		<Object>
	targetSelector:		<Object>

allowedInStyle
	- xxxxx

description
	- xxxxx

dialog
	- xxxxx

icon
	- xxxxx

label
	- xxxxx

name
	- xxxxx

order
	- xxxxx

ordering
	- xxxxx

precondition
	- xxxxx

target
	- xxxxx

targetSelector
	- xxxxx
!

!PragmaSetting methodsFor: 'comparing' stamp: 'alain.plantec 4/20/2009 15:18'!
= other 
	^ self species = other species
		and: [self name = other name]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 13:58'!
allowedInStyle
	^ allowedInStyle ifNil: [allowedInStyle := self hasValue]! !

!PragmaSetting methodsFor: 'user interface' stamp: 'alain.plantec 3/11/2009 15:38'!
asString
	^ self name! !

!PragmaSetting methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:13'!
chooseFileDirectory
	| result |
	result := self theme chooseDirectoryIn: World title: 'Choose a file' path: nil.
	result ifNotNil: [ self realValue: result fullName ].! !

!PragmaSetting methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:13'!
chooseFilePath
	| result |
	result := self theme fileOpenIn: World title: 'Choose a file' extensions: nil path: nil preview: true.
	result 
		ifNotNil: [
			self realValue: result name. 
			result close]! !

!PragmaSetting methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:13'!
defaultIcon
	^ self theme smallConfigurationIcon! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/13/2009 15:52'!
description
	"Answer the value of description"

	^ description ifNil: [description := '']! !

!PragmaSetting methodsFor: 'accessing' stamp: 'alain.plantec 3/25/2009 19:36'!
description: aText
	"Set the value of description"

	description := aText! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/30/2010 08:07'!
dialog
	^ dialog! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/30/2010 07:59'!
dialog: aValuable
	dialog := aValuable! !

!PragmaSetting methodsFor: 'user interface' stamp: 'alain.plantec 3/12/2009 12:58'!
enabled
	^ true! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:12'!
fontButtonLabel
	| emphases |
	emphases := IdentityDictionary new at: nil put: 'Regular' translated;
				 at: 0 put: 'Regular' translated;
				 at: 1 put: 'Bold' translated;
				 at: 2 put: 'Italic' translated;
				 at: 4 put: 'Underlined' translated;
				 at: 8 put: 'Narrow' translated; 
				 at: 16 put: 'StruckOut' translated;
				 yourself.
	^ StringMorph contents: self realValue familyName , ' '
			, (emphases
					at: self realValue emphasis
					ifAbsent: ['']) , ' ' , self realValue pointSize asString font: self realValue! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:12'!
getFont
	^ FontChooser
		openWithWindowTitle: 'Font for ' , self name
		for: self
		setSelector: #realValue:
		getSelector: #realValue ! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 14:02'!
hasDefault
	^ false! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 14:03'!
hasEditableList
	^ false! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 14:03'!
hasValue
	^ false! !

!PragmaSetting methodsFor: 'comparing' stamp: 'alain.plantec 4/20/2009 15:19'!
hash
	^ self species hash bitXor: self name hash ! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 08:27'!
icon
	^ icon ifNil: [icon := self defaultIcon]! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 11/23/2009 22:18'!
icon: aForm 
	icon := aForm! !

!PragmaSetting methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:13'!
inputMorphFor: aContainer 
	^ self inputWidget
		ifNotNil: [:iw | 
			( self theme newRowIn: World for: {iw})
				 clipSubmorphs: true;
				 hResizing: #shrinkWrap;
				 cellInset: 0;
				 yourself]! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/30/2010 08:07'!
inputWidget
	"return the default widget for the input a the setting"
	^ self dialog
		ifNotNil: [:d | d numArgs = 0
				ifTrue: [d value]
				ifFalse: [d value: self]] ! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 14:11'!
label
	^ label ifNil: [self name] ifNotNil: [label value]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 14:11'!
label: aStringOrBlockOrMessageSend
	label := aStringOrBlockOrMessageSend! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 2/11/2011 14:09'!
labelMorphFor: aContainer
	^ StringMorph contents: (aContainer model viewedLabelOfSetting: self)! !

!PragmaSetting methodsFor: 'accessing' stamp: 'alain.plantec 10/18/2009 12:40'!
name
	^ name ifNil: [super name] ifNotNil: [name value]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'alain.plantec 10/18/2009 12:39'!
name: aNameOrBlockOrMessageSend
	name := aNameOrBlockOrMessageSend! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 08:18'!
noOrdering
	ordering := false! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:04'!
notInStyle
	allowedInStyle := false! !

!PragmaSetting methodsFor: 'accessing' stamp: 'alain.plantec 4/2/2009 11:59'!
order
	^ order! !

!PragmaSetting methodsFor: 'accessing' stamp: 'alain.plantec 4/2/2009 10:53'!
order: aNumber
	order := aNumber! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 08:18'!
ordering
	^ ordering ifNil: [ordering := true]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 08:42'!
precondition
	^ precondition ifNil: [precondition := [true]].! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 08:43'!
precondition: aValuable
	precondition := aValuable! !

!PragmaSetting methodsFor: 'accessing' stamp: 'lr 3/14/2010 21:13'!
realTarget
	^ target isSymbol
		ifTrue: [ Smalltalk globals at: target ifAbsent: [  ] ]
		ifFalse: [ target ]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:47'!
settingReceiver
	^ self targetSelector ifNil: [self realTarget] ifNotNil: [self realTarget perform: self targetSelector]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 12/2/2009 16:40'!
target
	^ target! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/26/2009 21:15'!
target: anObject
	target := anObject! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/26/2009 21:15'!
targetSelector
	^ targetSelector! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/26/2009 21:16'!
targetSelector: aSelector
	targetSelector := aSelector! !

!PragmaSetting methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:13'!
theme
	^ UITheme current ! !

PragmaSetting subclass: #SettingDeclaration
	instanceVariableNames: 'getSelector setSelector type default ghostHelp'
	classVariableNames: 'ValueListCache'
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!SettingDeclaration commentStamp: 'AlainPlantec 1/3/2011 10:52' prior: 0!
A SettingDeclaration is an handler for a writtable setting value. 
Its main responsibility is allow real value changing and to hold descriptive data about the setting. When the real value is changed, all declared listeners are notified. The standard event mechanism is used for that purpose. See #value: method which is call in order to change a setting value. Whatever object can declare its interest in the real value changes by sending #whenChangedSend:to: to a setting. See notification protocol.

For convenience, my olds values are registered in a list. This list is used by the SystemSettingBrowser in order to easily retrieve previous values.

The type instance variable is a Symbol. It is used by the SystemSettingBrowser in order to build the input widget of a setting.
It can be the name of the class or the name of a superclass of the value. In that case, the input widget is built by #settingInputWidgetForNode: implemented by the class which name is the value of the type instance variable. As an example, the input widget for a Boolean is built by Boolean class>>settingInputWidgetForNode:.

Type value is not limited to be a class name. It can also be whatever symbol. In that case, the selector corresponding to the message to send to the setting in order to build the inputWidget is built by SettingDeclaration>>localInputWidgetSelector. If the type instance variable is not set, then it is dynamically set to the class name of the value. 

See SettingManager comment for more explanations.

Instance Variables
	default:		<Object>
	getSelector:		<Object>
	ghostHelp:		<Object>
	setSelector:		<Object>
	type:		<Object>

default
	- xxxxx

getSelector
	- xxxxx

ghostHelp
	- xxxxx

setSelector
	- xxxxx

type
	- xxxxx
!

SettingDeclaration subclass: #MultipleSettingDeclaration
	instanceVariableNames: 'domainValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!MultipleSettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 9/3/2010 15:51'!
domainValues
	^ domainValues ifNil: [domainValues := OrderedCollection new]! !

!MultipleSettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 11/25/2009 22:09'!
domainValues: aCollection 
	domainValues := aCollection asArray collect: [ :v | v settingFixedDomainValueNodeFrom: self]! !

!MultipleSettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 22:09'!
domainValuesLabels
	^ self domainValues
		collect: [:f | f name]! !

!MultipleSettingDeclaration methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 9/3/2010 17:04'!
fixedDomainValueNodeForAssociation: anAssociation 
	| s |
	^ (s := (SettingDeclaration new name: anAssociation key))
		target: s;
		selector: #default;
		default: anAssociation value! !

!MultipleSettingDeclaration methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 9/3/2010 17:04'!
fixedDomainValueNodeForObject: anObject 
	| s |
	^ (s := (SettingDeclaration new name: anObject asString))
		target: s;
		selector: #default;
		default: anObject! !

!MultipleSettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 22:09'!
hasEditableList
	^ false! !

MultipleSettingDeclaration subclass: #PickOneSettingDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!PickOneSettingDeclaration commentStamp: 'AlainPlantec 1/3/2011 10:53' prior: 0!
A CheckListSettingDeclaration is a setting for which the value domain is a list. Each element of the list is an instance of FixedSettingValue. If domainValues is set, then the list of valid values is constant (initialized at declaration time). Instead, if getter is set, then the setting list is always dynamically computed. See SettingManager comment for more explanations. 

Instance Variables
!

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 11/30/2009 14:23'!
chooseValue
	| chosen |
	chosen := UIManager default
				chooseFrom: self domainValuesLabels
				values: self domainValues
				title: self label translated, ' choices' translated.
	chosen
		ifNotNil: [self realValue: chosen realValue]! !

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 10/17/2009 23:47'!
content
	^ (self domainValues
		detect: [:v | v realValue = self realValue]
		ifNone: [^ 'nil']) name! !

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 9/3/2010 11:19'!
defaultValue
	^ self default value ifNil: [self domainValues first realValue]
! !

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 10/17/2009 23:44'!
index
	self realValue ifNil: [self realValue: self default].
	^ self domainValues indexOf: (self domainValues detect: [:dv | dv realValue = self realValue] ifNone: [^ 0])! !

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 10/17/2009 23:39'!
index: anInteger
	self realValue: (self domainValues at: anInteger) realValue.
! !

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:11'!
inputWidget
	| widget row |
	row := self theme newRowIn: World for: {
				widget := (self theme 
						newDropListIn: World
						for: self
						list: #domainValuesLabels
						getSelected: #index
						setSelected: #index:
						getEnabled: #enabled
						useIndex: true
						help: self description)
						extent: 1 @ 30}.
	widget hResizing: #rigid.
	widget width: (self maxNameWidthForFont: widget font) + 50.
	^ row! !

SettingDeclaration subclass: #RangeSettingDeclaration
	instanceVariableNames: 'range'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!RangeSettingDeclaration commentStamp: 'alain.plantec 4/7/2009 23:03' prior: 0!
A RangeSettingValue is a setting for which the value domain is an interval.  See SettingManager comment for more explanations.

Instance Variables
	range:		<Interval>

range
	- The interval which define the value domain
!

!RangeSettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 16:57'!
hasEditableList
	^ false! !

!RangeSettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:14'!
inputWidget
	^ (self theme newRowIn: World for: {(self theme 
							newTextEntryIn: World 
							for: self
							get: #realValue
							set: #realValue:
							class: (Smalltalk globals at: self type)
							getEnabled: #enabled
							help: nil) hResizing: #rigid;
							width: 80;
							wantsFrameAdornments: true;
							yourself. 
						(PluggableSliderMorph new max: self range last;
							 min: self range first;
							 quantum: self range increment;
							on: self
							getValue: #realValue
							setValue: #realValue:) hResizing: #spaceFill;
							 vResizing: #spaceFIll;
							 setBalloonText: self description;
							 minWidth: 300; height: 20;
							 yourself}) 
				cellInset: 10;
				yourself! !

!RangeSettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 3/12/2009 16:19'!
range
	^ range ! !

!RangeSettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 3/3/2009 09:38'!
range: anInterval
	range := anInterval! !

!RangeSettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:48'!
realValue: aValue
	aValue
		ifNil: [self settingReceiver perform: self setSelector with: nil]
		ifNotNil: [self settingReceiver perform: self setSelector with: ((aValue max: self range first) min: self range last)].
	self triggerEvent: self privateChangeEvent with: self
	! !

!SettingDeclaration class methodsFor: 'icon accessing' stamp: 'alain.plantec 4/9/2009 00:15'!
nullifyForm
	^ (Form
	extent: 16@16
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 463732736 4285815129 4285815129 4285815129 4285815129 4285815129 2292449280 463732736 0 0 0 0 0 0 0 1688469504 4285815129 4278190080 4278190080 4278190080 4278190080 4278190080 4280091395 4285815129 1688469504 0 0 0 0 0 1688469504 4285815129 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4285815129 1688469504 0 0 0 463732736 4285815129 4278190080 4294967295 4294967295 4278190080 4278190080 4278190080 4278190080 4294967295 4294967295 4278190080 4285815129 463732736 0 0 4285815129 4278190080 4278190080 4294967295 4294967295 4294967295 4278190080 4278190080 4294967295 4294967295 4294967295 4278190080 4285815129 4285815129 0 0 4285815129 4278190080 4278190080 4278190080 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4278190080 4278190080 4278190080 4285815129 0 0 4285815129 4278190080 4278190080 4278190080 4278190080 4294967295 4294967295 4294967295 4294967295 4278190080 4278190080 4278190080 4278190080 4285815129 0 0 4285815129 4278190080 4278190080 4278190080 4278190080 4294967295 4294967295 4294967295 4294967295 4278190080 4278190080 4278190080 4278190080 4285815129 0 0 4285815129 4278190080 4278190080 4278190080 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4278190080 4278190080 4278190080 4285815129 0 0 2292449280 4281927174 4280025345 4294967295 4294967295 4294967295 4278190080 4278190080 4294967295 4294967295 4294967295 4278190080 4283696903 4285815129 0 0 463732736 4285815129 4278190080 4294967295 4294967295 4278190080 4278190080 4278190080 4278190080 4294967295 4294967295 4278190080 4285815129 463732736 0 0 0 1688469504 4285815129 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4285815129 1688469504 0 0 0 0 0 1688469504 4285815129 4285815129 4278190080 4278190080 4278190080 4278190080 4283696646 4285815129 1688469504 0 0 0 0 0 0 0 463732736 2292449280 4285815129 4285815129 4285815129 4285815129 2292449280 463732736 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)! !

!SettingDeclaration class methodsFor: 'icon accessing' stamp: 'alain.plantec 4/8/2009 17:40'!
smallCautionForm
	^ (Form
	extent: 16@16
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 463732736 2292449280 3517186048 4154720256 4154720256 3517186048 2292449280 463732736 0 0 0 0 0 0 0 1688469504 4137943040 4292614939 4293600292 4293534756 4293534499 4293402913 4292483352 4137943040 1688469504 0 0 0 0 0 1688469504 4290185224 4293600292 4292284175 4291428866 4291166208 4291100672 4291297794 4292087053 4293271070 4290184967 1688469504 0 0 0 463732736 4137943040 4293534756 4291823367 4291231744 4291166208 4294967295 4294967295 4290969600 4290969600 4291429637 4293073948 4137943040 463732736 0 0 2292449280 4292614682 4292284175 4291231744 4291035136 4291035136 4294967295 4294967295 4291035136 4291035136 4290838528 4291824395 4292351253 2292449280 0 0 3517186048 4293468963 4291363330 4291166208 4291035136 4291035136 4294967295 4294967295 4291035136 4291035136 4290707456 4290838785 4292942105 3517186048 0 0 4154720256 4293534242 4291166208 4291100672 4291035136 4291035136 4294967295 4294967295 4291035136 4290707456 4290641920 4290641920 4292942104 4154720256 0 0 4154720256 4293468449 4291035136 4291035136 4291035136 4291035136 4291100672 4291100672 4291035136 4290641920 4290576384 4290510848 4292876056 4154720256 0 0 3517186048 4293271328 4291166722 4290969600 4291035136 4291035136 4291035136 4291035136 4291035136 4291035136 4290510848 4290576641 4292744726 3517186048 0 0 2292449280 4292417303 4291955981 4290838528 4291035136 4291035136 4294967295 4294967295 4291035136 4291035136 4290445312 4291430665 4292088338 2292449280 0 0 463732736 4137943040 4293139484 4291298565 4290707456 4290641920 4294967295 4294967295 4290510848 4290379776 4290905092 4292678933 4137943040 463732736 0 0 0 1688469504 4290184967 4293007898 4291693323 4290707713 4290510848 4290445312 4290511105 4291365129 4292678932 4290053638 1688469504 0 0 0 0 0 1688469504 4137943040 4292219924 4292876311 4292876055 4292810518 4292744726 4292088081 4137943040 1688469504 0 0 0 0 0 0 0 463732736 2292449280 3517186048 4154720256 4154720256 3517186048 2292449280 463732736 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)! !

!SettingDeclaration class methodsFor: 'cache accessing' stamp: 'alain.plantec 9/18/2009 09:57'!
valueListCache
	^ ValueListCache ifNil: [ValueListCache := WeakKeyDictionary new]! !

!SettingDeclaration class methodsFor: 'cache accessing' stamp: 'AlainPlantec 1/29/2010 10:50'!
valueListFor: aSettingValue
	| targetEntry |
	targetEntry := self valueListCache at: aSettingValue settingReceiver ifAbsentPut: [IdentityDictionary new].
	^ targetEntry at: aSettingValue getSelector ifAbsentPut: [OrderedCollection new].! !

!SettingDeclaration methodsFor: 'comparing' stamp: 'AlainPlantec 12/2/2009 16:44'!
= other
	self class = other class
		ifFalse: [^ false].
	self target == other target ifFalse: [^ false].
	self targetSelector = other targetSelector ifFalse: [^ false].
	self getSelector = other getSelector ifFalse: [^ false].
	self setSelector = other setSelector ifFalse: [^ false].
	^ true
	
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/9/2009 09:41'!
addToList: anItem
	(self list includes: anItem)
		ifFalse: [self list size > self maxKept
				ifTrue: [self list removeFirst].
			self list add: anItem.
			self changed: #list]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 3/11/2009 19:40'!
default
	^ default ! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 09:13'!
default: anObject 
	"initialize the default value"
	default := anObject.
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:17'!
defaultIcon
	^ (self type = #LogicalFont or: [self realValue isKindOf: AbstractFont])
		ifTrue: [ self theme   smallFontsIcon]
		ifFalse: [ self theme   smallPaintIcon]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 3/4/2009 11:48'!
defaultValue
	^ default value! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/19/2009 09:18'!
emptyList
	self list removeAll.
	self value ifNotNil: [self addToList: self realValue].! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 22:35'!
getSelector
	^ getSelector ifNil: [self name]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 23:07'!
getSelector: aSymbol
	getSelector := aSymbol! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 23:52'!
ghostHelp
	^ ghostHelp ! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 23:52'!
ghostHelp: aString
	ghostHelp := aString! !

!SettingDeclaration methodsFor: 'testing' stamp: 'alain.plantec 4/8/2009 16:59'!
hasDefault
	^ true! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 3/7/2010 14:34'!
hasEditableList
	^ (self realValue isKindOf: Boolean) not! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 16:50'!
hasPrivateListener
	^ self hasActionForEvent: self privateChangeEvent! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 11/25/2009 08:22'!
hasValue
	^ true! !

!SettingDeclaration methodsFor: 'comparing' stamp: 'AlainPlantec 12/13/2009 08:55'!
hash
	"hash is re-implemented because #= is re-implemented"
	^ self target hash
		bitXor: (self targetSelector hash
				bitXor: (self getSelector hash bitXor: self setSelector hash)) ! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/18/2009 23:20'!
index
	^ self list
		indexOf: self realValue
		ifAbsent: [self realValue
				ifNil: [0]
				ifNotNil: [self addToList: self realValue.
					self list size]]! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/18/2009 09:44'!
index: anInteger 
	self realValue: (self list at: anInteger)! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 5/20/2009 12:24'!
inputMorphFor: aContainer
	^ (super inputMorphFor: aContainer)
		ifNotNil: [:im |			
				self whenPrivateChangedSend: #settingValueChanged: to: aContainer model.
				self whenPrivateChangedSend: #innerWidgetChanged to: aContainer.
				im]! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:17'!
inputWidget
	"return the default widget for the input a the setting"
	| inputWidget xtraDialogWidget | 
	xtraDialogWidget := super inputWidget.
	inputWidget :=  self localInputWidgetSelector ifNotNil: [:localSel | self perform: localSel].
	(inputWidget isNil and: [self typeClass notNil])
		ifTrue: [inputWidget := (self inputWidgetSelectorInClass: self typeClass)
			ifNotNil: [:selector | self typeClass perform: selector with: self]].
	inputWidget
		ifNotNil: [(inputWidget respondsTo: #model:) 
			ifTrue: [inputWidget model: self]]. 
	xtraDialogWidget 
		ifNotNil: [inputWidget := inputWidget ifNil: [xtraDialogWidget] 
			ifNotNil: [ self theme  newRowIn: World for: {inputWidget. xtraDialogWidget}]].
	^ inputWidget ! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:17'!
inputWidgetForBoolean
	^  self theme  
			newCheckboxIn: World
			for: self
			getSelected: #realValue
			setSelected: #realValue:
			getEnabled: #enabled
			label: '' 
			help: nil! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:16'!
inputWidgetForColor
	^ (  
	 self theme 
		newColorChooserIn: World
		for: self
		getColor: #realValue
		setColor: #realValue:
		getEnabled: #enabled 
		help: 'Choose a color') minWidth: 150; width: 150; yourself! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:17'!
inputWidgetForFileOrDirectoryWithAction: aSymbol
	^  self theme  newRowIn: World for: {
					(EditableDropListMorph
						on: self
						list: #list
						selected: #index
						changeSelected: #index:
						useIndex: true 
						addToList: #addToList:
						class: String
						getEnabled: #enabled
						default: self defaultValue)
						ghostText: self ghostHelp;
						width: 450;
						wantsFrameAdornments: true;
						yourself.
					(  self theme  
						newButtonIn: World
						for: self
						getState: nil
						action: aSymbol
						arguments: nil
						getEnabled: nil
						label:  (AlphaImageMorph new image:  self theme  smallOpenIcon)
						help: 'Open directory chooser dialog' translated)
					}
					! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:17'!
inputWidgetForFont
	^ (  self theme 
		newButtonIn: World
		for: self
		getState: nil 
		action: #getFont
		arguments: {}
		getEnabled: #enabled
		getLabel: #fontButtonLabel
		help: 'Choose a font ...' translated) cornerStyle: #square
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 9/17/2011 15:39'!
inputWidgetForLabel
	| widget |
	widget := self theme newLabelIn: World for: self label: '' getEnabled: nil.
	widget getTextSelector: #preferenceValue.
	widget width: 450.
	^ widget! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:20'!
inputWidgetForNumber
	^ self inputWidgetForString width: 200.
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:16'!
inputWidgetForPassword
	| widget |
	widget :=  self theme  
				newTextEntryIn: World
				for: self
				get: #realValue
				set: #realValue:
				class: String
				getEnabled: #enabled
				help: nil.
	widget
		wantsFrameAdornments: true;
		hResizing: #rigid;
		width: 450; 
		font: (StrikeFont passwordFontSize:  self theme  textFont pointSize).
	^ widget! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:20'!
inputWidgetForPoint
	^ self inputWidgetForString width: 200.
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'lr 3/14/2010 21:13'!
inputWidgetForString
	| widget |
	widget := EditableDropListMorph
		on: self
		list: #list
		selected: #index
		changeSelected: #index:
		useIndex: true
		addToList: #addToList:
		class: (Smalltalk globals at: self type)
		getEnabled: #enabled
		default: self defaultValue.
	widget ghostText: self ghostHelp.
	widget contentMorph wantsFrameAdornments: true.
	widget width: 450.
	^ widget! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:21'!
inputWidgetSelectorInClass: aClass 
	| selector |
	^ (aClass respondsTo: (selector := #settingInputWidgetForNode:))
		ifTrue: [selector] ! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/18/2009 23:10'!
list
	^ self class valueListFor: self! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/17/2009 21:20'!
listenersForEvent: anEvent
	| listeners actionSequence |	
	listeners := Set new.	
	actionSequence := self actionMap at: anEvent ifAbsent: [].	
	actionSequence ifNotNil: [
		actionSequence isCollection	 
			ifTrue: [actionSequence do: [:oneAction | 
					oneAction receiver ifNotNil: [:r | listeners add: r]]]
			ifFalse: [listeners add: actionSequence receiver]].
	^ listeners! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:18'!
localInputWidgetSelector
	| imSel |
	^ (self respondsTo: (imSel := ('inputWidgetFor' , self type) asSymbol)) ifTrue: [imSel]! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/6/2009 21:54'!
maxKept
	"maximum number of old values which are kept in
	list instance variable"
	^ 5! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 21:35'!
maxNameWidthForFont: aFont 
	| max |
	max := 0.
	self domainValuesLabels
		do: [:v | max := max max: (aFont widthOfStringOrText: v)].
	^ max ! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 9/14/2011 16:37'!
preferenceValue
	^ self realValue! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 4/8/2009 15:59'!
privateChangeEvent
	^ #privateSettingValueChanged! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 16:10'!
privateForget: aReceiver 
	"do not notify aReceiver anymore"
	self removeActionsWithReceiver: aReceiver forEvent: self privateChangeEvent! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 16:50'!
privateForget: aReceiver selector: aSelector 
	"Private"
	self
		removeActionsSatisfying: [:action | action receiver == aReceiver
				and: [action selector = aSelector]]
		forEvent: self privateChangeEvent! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 16:35'!
privateListeners
	^ self listenersForEvent: self privateChangeEvent! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:49'!
realValue
	"Answer the value of realValue"

	^ (self settingReceiver perform: self getSelector) 
		ifNil: [self setToDefault. 
			self settingReceiver perform: self getSelector]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:49'!
realValue: anObject
	"Set the value of realValue"
	self settingReceiver perform: self setSelector with: anObject.
	self triggerEvent: self privateChangeEvent with: self.! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 13:49'!
selector: aSymbol
	self getSelector: aSymbol.
	self setSelector: (aSymbol, ':') asSymbol.! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 22:36'!
setSelector
	^ setSelector ifNil: [self getSelector ifNotNil: [:gs | (gs, ':') asSymbol]]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 23:07'!
setSelector: aSymbol
	setSelector := aSymbol! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/18/2009 09:23'!
setToDefault
	self realValue: self defaultValue.
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 9/14/2011 16:47'!
settingStoreOn: aStream
	aStream tab;
		nextPutAll: (self target isSymbol
				ifTrue: [self target]
				ifFalse: [self target name]);
		nextPutAll: (self targetSelector
				ifNil: ['']
				ifNotNil: [' ' , self targetSelector]);
		 nextPutAll: ' ' , self setSelector , ' '.
	self preferenceValue settingStoreOn: aStream.
	aStream nextPut: $.;
		 cr! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 09:24'!
type
	"Answer the value of type"

	^ type ifNil: [type := self realValue class name]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 3/11/2009 18:22'!
type: anObject
	"Set the value of type"

	type := anObject! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'lr 3/14/2010 21:13'!
typeClass
	^ Smalltalk globals at: self type ifAbsent: [  ]! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 15:59'!
whenPrivateChangedSend: zeroOrOneArgSelector to: aSubscriber 
	"private, use #whenChangedSend:to: instead"
	self
		when: self privateChangeEvent
		send: zeroOrOneArgSelector
		to: aSubscriber! !

PragmaSetting subclass: #SystemSettingLauncher
	instanceVariableNames: 'script'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!SystemSettingLauncher methodsFor: 'accessing' stamp: 'FernandoOlivero 4/12/2011 10:32'!
defaultIcon
	^ self theme  smallDoItIcon! !

!SystemSettingLauncher methodsFor: 'accessing' stamp: 'FernandoOlivero 4/12/2011 10:45'!
inputWidget
	^ (self theme  
				newButtonIn: World 
				for: self
				getState: nil
				action: #launch
				arguments: {}
				getEnabled: nil
				getLabel: nil
				help: self description)
		 		label: ( self theme  newRowIn: World for: {AlphaImageMorph new image: self theme smallRedoIcon. 
				self theme buttonLabelForText: 'Launch' translated});
		 		yourself.! !

!SystemSettingLauncher methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:51'!
launch
	self script 
			ifNotNil: [:s | s isSymbol 
				ifTrue: [s numArgs = 0 
					ifTrue: [self settingReceiver perform: s]
					ifFalse: [self settingReceiver perform: s with: self]]
				ifFalse: [s numArgs = 0 
					ifTrue: [s value]
					ifFalse: [s value: self]]]! !

!SystemSettingLauncher methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 13:22'!
script
	^ script ifNil: [script := []]! !

!SystemSettingLauncher methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 14:19'!
script: aValuableOrSymbol
	script := aValuableOrSymbol! !

Object subclass: #SettingFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Filter'!

!SettingFilter commentStamp: 'alain.plantec 4/8/2009 12:51' prior: 0!
A SettingFilter is a filter wich is used by a SettingBrowser in order to select which nodes of the setting trees are to be shown. A SettingFilter subclass must redefine the #keepHandler: method which return true if the argument handler is to be kept.

Instance Variables
!

!SettingFilter methodsFor: 'filtering' stamp: 'alain.plantec 9/18/2009 23:46'!
keepSettingTreeNode: aSettingNode 
	^ false! !

SettingFilter subclass: #SettingPackageFilter
	instanceVariableNames: 'packages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Filter'!

!SettingPackageFilter commentStamp: 'AlainPlantec 12/31/2010 11:38' prior: 0!
A SettingPackageFilter is a SettingFilter used to select settings from a given list of packages: a setting is kept only if the package in which it is defined is present in the given package list

Instance Variables
	packages:		<Collection>

packages
	- The collection of PackageInfo the filter is currently using!

!SettingPackageFilter methodsFor: 'filtering' stamp: 'AlainPlantec 11/27/2009 10:28'!
keepSettingTreeNode: aSettingNode 
	| rp |
	^ (rp := aSettingNode receiverPackage) notNil and: [self packages includes: rp]! !

!SettingPackageFilter methodsFor: 'accessing' stamp: 'alain.plantec 3/16/2009 23:00'!
packages
	"Answer the value of packages"

	^ packages! !

!SettingPackageFilter methodsFor: 'accessing' stamp: 'alain.plantec 4/8/2009 12:55'!
packages: aCollectionOfPackageInfo
	"Set the value of packages"

	packages := aCollectionOfPackageInfo
! !

SettingFilter subclass: #SettingRegexpFilter
	instanceVariableNames: 'regexp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Filter'!

!SettingRegexpFilter commentStamp: 'alain.plantec 4/8/2009 13:02' prior: 0!
A SettingRegexpFilter is SettingFilter which makes use of a regular expression in order to keep or not a SettingHandler.

Instance Variables
	regexp:		<String>

regexp
	- a regular expression string
!

!SettingRegexpFilter methodsFor: 'filtering' stamp: 'alain.plantec 9/18/2009 23:43'!
keepSettingDeclaration: aSettingDeclaration
	^ ({aSettingDeclaration name. aSettingDeclaration description}
				collect: [:e | e ifNil: ['']])
				anySatisfy: [:token | token substrings anySatisfy: [:sub | sub matchesRegexIgnoringCase: self regexp]]
! !

!SettingRegexpFilter methodsFor: 'filtering' stamp: 'alain.plantec 9/18/2009 23:46'!
keepSettingTreeNode: aSettingNode 
	^ (({aSettingNode methodClass name. aSettingNode selector}
			collect: [:e | e ifNil: ['']])
			anySatisfy: [:token | token substrings anySatisfy: [:sub | sub matchesRegexIgnoringCase: self regexp]])
		or: [self keepSettingDeclaration: aSettingNode settingDeclaration] ! !

!SettingRegexpFilter methodsFor: 'accessing' stamp: 'alain.plantec 4/7/2009 23:52'!
regexp
	"Answer the value of regexp"

	^ regexp! !

!SettingRegexpFilter methodsFor: 'accessing' stamp: 'alain.plantec 4/7/2009 23:52'!
regexp: anObject
	"Set the value of regexp"

	regexp := anObject! !

SettingFilter subclass: #SettingTokenFilter
	instanceVariableNames: 'token'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Filter'!

!SettingTokenFilter commentStamp: 'alain.plantec 4/8/2009 13:03' prior: 0!
A SettingTokenFilter is SettingFilter which makes use of a token (substring) in order to keep or not a SettingHandler.


Instance Variables
	token:		<String>

!

!SettingTokenFilter methodsFor: 'filtering' stamp: 'AlainPlantec 9/24/2011 22:15'!
keepSettingDeclaration: aSettingDeclaration
	| selector |
	selector := (aSettingDeclaration respondsTo: #getSelector) ifTrue: [aSettingDeclaration getSelector] ifFalse: [''].
	^ (({aSettingDeclaration name. 
		aSettingDeclaration label. 
		selector asUppercase. 
		aSettingDeclaration description}
				collect: [:e | (e ifNil: ['']) asUppercase])
				includesSubstringAnywhere: self token asUppercase)
! !

!SettingTokenFilter methodsFor: 'filtering' stamp: 'AlainPlantec 9/24/2011 22:18'!
keepSettingTreeNode: aSettingNode 
	^ (({aSettingNode methodClass name asUppercase. aSettingNode selector asUppercase}
			collect: [:e | e ifNil: ['']])
			includesSubstringAnywhere: self token asUppercase)
		or: [self keepSettingDeclaration: aSettingNode settingDeclaration]! !

!SettingTokenFilter methodsFor: 'accessing' stamp: 'alain.plantec 3/16/2009 22:51'!
token
	"Answer the value of token"

	^ token! !

!SettingTokenFilter methodsFor: 'accessing' stamp: 'alain.plantec 3/16/2009 22:51'!
token: aString
	"Set the value of token"

	token := aString! !

Object subclass: #SettingNodeBuilder
	instanceVariableNames: 'node builder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Browser'!

!SettingNodeBuilder methodsFor: 'private tree building' stamp: 'AlainPlantec 11/24/2009 13:34'!
asParentWhile: aBlock
	builder parent: self while: aBlock
! !

!SettingNodeBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 13:33'!
builder: aTreeBuilder
	builder := aTreeBuilder! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:50'!
default: anObject
	node item  default: anObject
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
description: aText
	node item description: aText! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
dialog: aSymbol
	node item dialog: aSymbol
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
domainValues: aCollection 
	node item domainValues: aCollection 
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
getSelector: aSymbol
	node item getSelector: aSymbol
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
ghostHelp: aString
	node item ghostHelp: aString
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
icon: aForm
	node item icon: aForm! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
label: aStringOrBlockOrMessageSend
	node item label: aStringOrBlockOrMessageSend.
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 14:02'!
name
	^ node item name
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
name: aStringOrBlockOrMessageSend
	node item name: aStringOrBlockOrMessageSend.
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/27/2009 23:26'!
noOrdering
	node item noOrdering
! !

!SettingNodeBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 12:43'!
node
	^ node! !

!SettingNodeBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 12:43'!
node: aSettingTreeNode
	node := aSettingTreeNode! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 12/3/2009 16:21'!
notInStyle
	node item notInStyle! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
order: aNumber
	node item order: aNumber
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
parent: aSymbol
	node parentName: aSymbol! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
parentName
	^ node parentName
	! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
parentName: aSymbol
	node parentName: aSymbol.
	! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 12/5/2009 08:44'!
precondition: aValuable
	node item precondition: aValuable
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
range: anInterval
	node item range: anInterval
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 12/5/2009 13:52'!
script: aSymbol
	node item script: aSymbol
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
selector: aSymbol
	node item selector: aSymbol
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
setSelector: aSymbol
	node item setSelector: aSymbol
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
target: anObject
	node item target: anObject
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/26/2009 21:39'!
targetSelector: aSelector
	node item targetSelector: aSelector
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
type: anObject
	node item type: anObject
! !

!SettingNodeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 11/24/2009 12:40'!
with: aBlock
	self asParentWhile: aBlock! !

Object subclass: #SettingTree
	instanceVariableNames: 'collector nodeList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Browser'!

!SettingTree commentStamp: 'AlainPlantec 1/2/2011 22:56' prior: 0!
A SettingTree is responsible for the building of system setting trees from a collection of pragma.
Built tree nodes are instances of SettingNode. Built trees are suitable for tree browsing with the help of a MorphTreeMorph (see SettingBrowser>>treeMorphIn: and SettingBrowser>>roots to know how a SettingBrowser is initializing its MorphTreeMorph with the help of a SettingTree). 

Below, two examples of tree building with a SettingCollector are shown.
---------------
(SettingTree acceptableKeywords: #(#'systemsettings')) settingTreeRoots. "get system setting trees"
---------------

Instance Variables
	collector		<PragmaCollector>
	nodeList		<A list of SettingNode>
!

!SettingTree class methodsFor: 'instance creation' stamp: 'AlainPlantec 1/2/2011 22:55'!
acceptableKeywords: aCollectionOfStrings
	^ self new acceptableKeywords: aCollectionOfStrings 
! !

!SettingTree methodsFor: 'accessing' stamp: 'asasa  4/28/2010 18:33'!
acceptableKeywords: keywords
	self collector filter: [:prg | prg methodClass isMeta and: [keywords includes: prg keyword]].
	nodeList := nil.
	self collector reset.
! !

!SettingTree methodsFor: 'checking' stamp: 'FernandoOlivero 4/12/2011 10:17'!
checkForUnknownParent
	self unclassified ifNotEmpty: [:nodesWithUnknownParent | | builder g |
		builder := SettingTreeBuilder new.
		g := (builder group: #nodesWithUnknownParent) node.
		self nodeList addFirst: g.
		g model: self.
		g item label: '*** Unclassified ***'.
		g item icon:  self theme  smallDebugIcon.
		g item order: 0.0.
		nodesWithUnknownParent do: [:node | node parentName: #nodesWithUnknownParent]]! !

!SettingTree methodsFor: 'checking' stamp: 'AlainPlantec 12/7/2009 08:56'!
checkForUnknownTarget
	"check only root because other nodes with unknown target inherits it from its parent"
	self settingTreeRoots do: [:node | node item target ifNil: [node item target: (node pragma ifNotNil: [:prg | prg methodClass theNonMetaClass])]].
	self settingTreeRoots do: [:node | node checkForUnknownTarget].
! !

!SettingTree methodsFor: 'querying' stamp: 'MarcusDenker 5/31/2011 15:34'!
childrenOf: aNode 
	| children  idx order |
	"Get aNode children. Assign children order if one child has an order which is not nil"
	children := self nodeList select: [:p | p parentName = aNode settingDeclaration name].
	(children detect: [:n | n order notNil] ifNone:[])  ifNotNil: [:firstWithOrder |
		idx := children indexOf: firstWithOrder.
		order := firstWithOrder order.
		idx > 1
			ifTrue: [(idx - 1 to: 1) 
				do: [:pos | 
					(children at: pos) order: order - 1. 
					order := order - 1]].
		order := firstWithOrder order.
		idx + 1 to: children size
			do: [:pos | 
				(children at: pos) order 
					ifNil:  [(children at: pos) order: order + 1] 
					ifNotNil: [order := (children at: pos) order].
				order := order + 1]].
	^ children! !

!SettingTree methodsFor: 'accessing' stamp: 'alain.plantec 10/19/2009 11:28'!
collector
	^ collector ifNil: [collector := PragmaCollector new]! !

!SettingTree methodsFor: 'querying' stamp: 'alain.plantec 9/19/2009 09:32'!
deeplyDetect: aBlock 
	self settingTreeRoots
		do: [:sub | (sub deeplyDetect: aBlock)
				ifNotNil: [:found | ^ found]].
	^ nil ! !

!SettingTree methodsFor: 'querying' stamp: 'alain.plantec 4/3/2009 07:53'!
deeplySelect: aBlock
	^ self deeplySelect: aBlock in: OrderedCollection new
! !

!SettingTree methodsFor: 'querying' stamp: 'alain.plantec 4/13/2009 08:04'!
deeplySelect: aBlock in: aCollection 
	self settingTreeRoots
		do: [:aRoot | aRoot deeplySelect: aBlock in: aCollection]. 
	^ aCollection! !

!SettingTree methodsFor: 'private tree building' stamp: 'AlainPlantec 11/27/2009 22:05'!
emptyNodeNamed: aSymbol
	(self nodeNamed: aSymbol)
! !

!SettingTree methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 13:52'!
itemSortBlock
	^ [:a :b | 
		((a order notNil and: [b order notNil]) and: [a order ~= b order])
			ifTrue: [a order < b order]
			ifFalse: [((a order isNil and: [b order isNil]) or: [a order = b order])
				ifTrue: [a label < b label]
				ifFalse: [a order ifNil: [false] ifNotNil: [true]]]]! !

!SettingTree methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 08:46'!
nodeList
	| builder |
	^ nodeList 
		ifNil: [
			builder := SettingTreeBuilder new.
			self pragmasDo: [:p | builder buildPragma: p].
			nodeList := builder nodeList.
			nodeList do: [:n | n model: self].
			self checkForUnknownParent.
			self checkForUnknownTarget.
			self nodeList]! !

!SettingTree methodsFor: 'private tree building' stamp: 'AlainPlantec 11/24/2009 09:35'!
nodeNamed: aSymbol
	^ self nodeNamed: aSymbol ifAbsent: []
! !

!SettingTree methodsFor: 'private tree building' stamp: 'AlainPlantec 11/24/2009 09:34'!
nodeNamed: aSymbol ifAbsent: aBlock
	^ self nodeList detect: [:d | d settingDeclaration name = aSymbol] ifNone: aBlock! !

!SettingTree methodsFor: 'querying' stamp: 'alain.plantec 10/19/2009 13:56'!
pragmasDo: aBlock
	^  self collector do: aBlock! !

!SettingTree methodsFor: 'querying' stamp: 'AlainPlantec 12/5/2009 08:51'!
retainedNodesFromList: aListOfNodes
	| retained |
	retained := OrderedCollection new.
	aListOfNodes do: [:n | [n item precondition value ifTrue: [retained add: n]] on: Error do: [retained add: n]].
	^ retained! !

!SettingTree methodsFor: 'querying' stamp: 'AlainPlantec 12/5/2009 08:51'!
settingTreeRoots
	^ self retainedNodesFromList: ((self nodeList select: [:n | n parentName isNil]) asArray sort: self sortBlock) asOrderedCollection.
! !

!SettingTree methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 13:52'!
sortBlock
	^ [:a :b | self itemSortBlock value: a settingDeclaration value: b settingDeclaration]! !

!SettingTree methodsFor: 'theme' stamp: 'BenjaminVanRyseghem 5/16/2011 16:12'!
theme

	^ UITheme current! !

!SettingTree methodsFor: 'querying' stamp: 'AlainPlantec 11/24/2009 12:01'!
unclassified
	^ self nodeList select: [:node | node parentName notNil and: [node parentNode isNil]].! !

!SettingTree methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 11:51'!
updateList
	nodeList := nil.
! !

!SettingTree methodsFor: 'updating' stamp: 'alain.plantec 10/19/2009 11:37'!
whenChangedSend: aSelector to: aTarget
	self collector whenChangedSend: aSelector to: aTarget.
	self collector whenResetSend: aSelector to: aTarget.! !

Object subclass: #SettingTreeBuilder
	instanceVariableNames: 'nodeList currentParent currentPragma'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Browser'!

!SettingTreeBuilder class methodsFor: 'instance creation' stamp: 'AlainPlantec 11/24/2009 11:53'!
acceptableKeywords: aCollectionOfRegExps 
	^ self new acceptableKeywords: aCollectionOfRegExps 
! !

!SettingTreeBuilder methodsFor: 'accessing' stamp: 'asasa  4/28/2010 18:21'!
buildPragma: aPragma
	currentPragma := aPragma.
	self perform: aPragma keyword withArguments: aPragma arguments.
	^ nodeList! !

!SettingTreeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 1/29/2010 14:07'!
group: aSymbol
	^ self nodeClass: PragmaSetting name: aSymbol
! !

!SettingTreeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 12/5/2009 13:50'!
launcher: aSymbol
	^ self nodeClass: SystemSettingLauncher name: aSymbol
! !

!SettingTreeBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 1/30/2010 11:53'!
nodeClass
	^ SettingNode! !

!SettingTreeBuilder methodsFor: 'private tree building' stamp: 'AlainPlantec 1/30/2010 11:54'!
nodeClass: aClass name: aSymbol
	| node |
	node := self nodeClass with: aClass new.
	node item name: aSymbol.
	node pragma: currentPragma.
	node parentName: (currentParent ifNotNil: [currentParent name]).
	self nodeList add: node.
	^ (SettingNodeBuilder new) node: node; builder: self; yourself! !

!SettingTreeBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 12:05'!
nodeList
	^ nodeList ifNil: [nodeList := OrderedCollection new]
! !

!SettingTreeBuilder methodsFor: 'private tree building' stamp: 'AlainPlantec 11/24/2009 13:35'!
parent: aNode while: aBlock
	| oldParent |
	oldParent := currentParent.
	currentParent := aNode.
	aBlock value.
	currentParent := oldParent! !

!SettingTreeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 12/2/2009 11:47'!
pickOne: aSymbol
	^ self nodeClass: PickOneSettingDeclaration name: aSymbol
! !

!SettingTreeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 11/24/2009 12:15'!
range: aSymbol
	^ self nodeClass: RangeSettingDeclaration name: aSymbol
! !

!SettingTreeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 11/24/2009 12:15'!
setting: aSymbol
	^ self nodeClass: SettingDeclaration name: aSymbol
! !

!SettingTreeBuilder methodsFor: 'pragmas' stamp: 'AlainPlantec 4/29/2010 17:44'!
systemsettings
	<settingPragmaProcessor>
	"Process a <systemsettings> pragma"
	currentPragma methodClass theNonMetaClass perform: currentPragma selector with: self.! !

!String class methodsFor: '*system-settings-browser' stamp: 'alain.plantec 3/18/2009 15:02'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForString! !
